package hu.elte.teszteszkozok.gyak3.stack;

import hu.elte.teszteszkozok.gyak3.stack.StackEmptyException;
import hu.elte.teszteszkozok.gyak3.stack.StackOverflowException;
import java.util.*;
import java.io.*;

public class Stack<T> implements Serializable {

    private final Object[] elements;
    private int top = -1;

    public Stack(int capacity) {
        if (capacity < 0) {
            throw new IllegalArgumentException("capacity must be greater than or equal to zero");
        }
        elements = new Object[capacity];
    }

    public void push(T element) throws StackOverflowException {
        if (top == elements.length - 1) {
            throw new StackOverflowException(String.format("maximum capacity of stack is %d, it contains %d element(s)", elements.length, top + 1));
        }
        elements[++top] = element;
    }

    public T pop() throws StackEmptyException {
        if (top == -1) {
            throw new StackEmptyException("stack is empty, no element to pop");
        }

        T element = (T) elements[top--];
        elements[top + 1] = null;
        return element;
    }

    public boolean isEmpty() {
        return top == -1;
    }

    public boolean isFull() {
        return top == elements.length - 1;
    }

    @Override
    public String toString() {
        return Arrays.toString(elements);
    }

}
